
 /*
  * ZFIND.C 
  */

/*
   Written by :
     harold barker
     Dark Side of the Moon SoftWare
     P.O. Box 6051
     Champaign, Illinois
     61826-6051
     
             : barker@MRCNeXT.cso.uiuc.edu
     BitNet  : barker@VME.cso.uiuc.edu
     FidoNet : harold barker fidonet>1:233/493
  
   Compiler : Lattice C 3.4, a fine C compiler backed by exelant support.
  
   History :
   0.01	02/16/1989	original coding, so i expect some bugs.
                        possible errors that i have not though of.
			Does not deal with bad archives well.
  
  
   This code is placed in the public domain.
  
   I do think it is kind of rude to charge any money for distribution
   someone else's work, so please do not.
  
   For more information read the ZIP format file provided by Phil.
*/

#include <DOS.H>
#include <Stdio.H>
#include <StdLib.H>
#include <fcntl.h>

#include <MyDef.H>
#include <MyLib.H>
#include <Archive.H>
#include <Author.H>

 /*
  * if TEST is defined test code will be included, if not defind code is sutable to place in library 
  */
/* #define TEST */

static ZIO      zio_b[MAX_ZIOB];		/* array of type ZIO */
static central_zip czta;			/* read buffer for centrial directory record */
static local_zip lzta;				/* read buffer for local directory record */
static int      _ZFIRST_ = TRUE;		/* TRUE until first call to zopen */

int             _ZERR = 0;			/* zip error flag */

void 
pecz(int);					/* print end_central_zip */
void 
pcz(central_zip *);				/* print central_zip */
void 
plz(local_zip *);				/* print local_zip */
void 
pzi(ZIPINFO *);					/* print ZINFO */


#ifdef TEST

main(argc, argv)
int             argc;
byte          **argv;
{
int             zfh;
ZIPINFO         zinfo;
int             error;

    cprintf("\nargv[1] -:%s\n", argv[1]);
    zfh = zopen(argv[1]);
    pecz(zfh);

    for (error = zfindfirst(zfh, "", &zinfo); error != -1; error = zfindnext(&zinfo))
	pzi(&zinfo);

    zfh = zclose(zfh);

}						/* end of main */

#endif


int
zfindfirst(zh, pattern, zinfo)
int             zh;				/* zipfile handle */
byte           *pattern;			/* pattern to match, ala grep */
ZIPINFO        *zinfo;				/* pointer to structure to return info in */
{
int             ret = 0;			/* return code */
int             i;				/* temp int */
int             size;				/* number of bytes to read */

    _ZERR = 0;					/* clear zip error flag */

    if (zio_b[zh].fh == -1) {			/* if file is closed */
	ret = 31;				/* set error code */
    }						/* end of if closed */

    if ((zh < -1) || (zh > MAX_ZIOB - 1)) {	/* if invalid handle */
	_ZERR = 6;				/* set _ZERR code */
	ret = -1;				/* set return code */
	goto done;				/* jump to exit point */
    }						/* end of if invalid zh */

 /*
  * read first central_zip record 
  */
    lseek(zio_b[zh].fh, zio_b[zh].start_cdir, 0);	/* seek to beginning of central directory */
    size = sizeof(central_zip);			/* set number of bytes to read */
    i = read(zio_b[zh].fh, (char *) &czta, size);	/* read first directory entry */
    if (i != size) {				/* if error reading */
	_ZERR = _OSERR;				/* save MsDos error code */
	ret = -1;				/* flag an error */
	poserr("read first cdir.");
	goto done;				/* jump to exit point */
    }						/* end of read error */

    if (czta.signature != CEN_SIG) {
	_ZERR = 31;				/* general error */
	ret = -1;				/* flag an error */
	goto done;
    }						/* end of if no signature found */

    zinfo->zptr = &zio_b[zh];			/* pointer to ZIO struct for this file */

    if (pattern != NULL)			/* if a pattern to match on was passed */
	stccpy(zinfo->pattern, pattern, PATTERN_SIZE);	/* copy */
    else
	zinfo->pattern[0] = EOS;

 /*
  * copy information to users data area 
  */
    zinfo->ver_made_by = czta.ver_made_by;
    zinfo->ver_to_extract = czta.ver_to_extract;
    zinfo->bit_flag = czta.bit_flag;
    zinfo->comp_method = czta.comp_method;
    zinfo->time = czta.time;
    zinfo->date = czta.date;
    zinfo->crc = czta.crc;
    zinfo->compressed_size = czta.compressed_size;
    zinfo->uncompressed_size = czta.uncompressed_size;
    zinfo->disk_num_start = czta.disk_num_start;/* number of disk holding this files beginning */
    zinfo->in_attrib = czta.in_attrib;		/* zip internal file attributes */
    zinfo->extern_attrib = czta.extern_attrib;	/* system dependent, low order byte is MsDos */
    zinfo->local_header_offset = czta.local_header_offset;	/* offset on first disk to local header */

    if (czta.name_len > 0) {			/* if name is present */
	size = (czta.name_len < (NAME_SIZE - 2)) ? czta.name_len : NAME_SIZE - 2;
	read(zio_b[zh].fh, (char *) zinfo->name, size);
	zinfo->name[size] = EOS;		/* add end of line char */
    }
    else {					/* name not resent, why i can not imagine */
	zinfo->name[0] = EOS;
    }

    if (czta.comment_len > 0) {			/* if comment present */
	size = (czta.comment_len < (COMMENT_SIZE - 2)) ? czta.comment_len : COMMENT_SIZE - 2;
	read(zio_b[zh].fh, (char *) zinfo->comment, size);
	zinfo->comment[size] = EOS;
    }
    else {					/* if no comment for this file */
	zinfo->comment[0] = EOS;
    }						/* end of if no comment */

    zinfo->next_offset = tell(zio_b[zh].fh);	/* offset of NEXT cdir */

#ifdef TEST
    pcz(&czta);
#endif

done:
    return (ret);
}						/* end of zfindfirst */




int
zfindnext(zinfo)
ZIPINFO        *zinfo;				/* pointer to structure to return info in */
{
int             ret = 0;			/* return code */
int             i;				/* temp int */
int             size;				/* number of bytes to read */

    _ZERR = 0;					/* clear zip error flag */
    if (zinfo->zptr->fh == -1) {		/* if file is closed */
	ret = 31;				/* set error code */
    }

 /*
  * read next central_zip record 
  */
    lseek(zinfo->zptr->fh, zinfo->next_offset, 0);	/* seek to beginning of central directory */
    size = sizeof(central_zip);
    i = read(zinfo->zptr->fh, (char *) &czta, size);	/* read directory entry */
    if (i != size) {				/* if error reading */
	_ZERR = _OSERR;				/* save MsDos error code */
	ret = -1;				/* flag an error */
	poserr("read");
	goto done;
    }						/* end of read error */

    if (czta.signature != CEN_SIG) {		/* not the beginning of a directory record */
	_ZERR = 31;				/* general error */
	ret = -1;				/* flag an error */
	goto done;				/* jump to exit point */
    }						/* end of if no signature found */

 /*
  * copy information to users data area 
  */
    zinfo->ver_made_by = czta.ver_made_by;
    zinfo->ver_to_extract = czta.ver_to_extract;
    zinfo->bit_flag = czta.bit_flag;
    zinfo->comp_method = czta.comp_method;
    zinfo->time = czta.time;
    zinfo->date = czta.date;
    zinfo->crc = czta.crc;
    zinfo->compressed_size = czta.compressed_size;
    zinfo->uncompressed_size = czta.uncompressed_size;
    zinfo->disk_num_start = czta.disk_num_start;/* number of disk holding this files beginning */
    zinfo->in_attrib = czta.in_attrib;		/* zip internal file attributes */
    zinfo->extern_attrib = czta.extern_attrib;	/* system dependent, low order byte is MsDos */
    zinfo->local_header_offset = czta.local_header_offset;	/* offset on first disk to local header */

    if (czta.name_len > 0) {			/* if there is a name to be read */
	size = (czta.name_len < (NAME_SIZE - 2)) ? czta.name_len : NAME_SIZE - 2;	/* make sure we do not read too much */
	read(zinfo->zptr->fh, (char *) zinfo->name, size);	/* the read */
	zinfo->name[size] = EOS;		/* there is not end of line in the zip file */
    }
    else {					/* no name to read */
	zinfo->name[0] = EOS;			/* clear any extraneous BS left form last call */
    }						/* end of else no file name this should generate an error */

    if (czta.comment_len > 0) {			/* if this file has a comment */
	size = (czta.comment_len < (COMMENT_SIZE - 2)) ? czta.comment_len : COMMENT_SIZE - 2;
	read(zinfo->zptr->fh, (char *) zinfo->comment, size);
	zinfo->comment[size] = EOS;
    }
    else {					/* if no file comment */
	zinfo->comment[0] = EOS;
    }						/* end of if no comment */

    zinfo->next_offset = tell(zinfo->zptr->fh);	/* offset of NEXT record  */

#ifdef TEST
    pcz(&czta);
#endif

done:
    return (ret);
}						/* end of zfindnext */



int
zopen(name)
byte           *name;
{
int             fh = -1;			/* local file handle */
int             i = 0;				/* VOLATILE counter */
int             ret = -1;			/* return code */
int             zh = -1;			/* zio_b index */
unsigned long   sig = 0L;			/* temp read buffer */
long            pos = 0L;			/* offset in to zip file */
int             size;				/* number of bytes to read */


    if (_ZFIRST_ == TRUE) {			/* if first time through */
	_ZFIRST_ = FALSE;			/* no longer the first time */
	_ZERR = 0;				/* clear error flag */
	for (i = 0; i < MAX_ZIOB; i++) {	/* for each element of the io_b array */
	    zio_b[i].name[0] = EOS;		/* no name */
	    zio_b[i].fh = -1;			/* file has not been opened */
	    zio_b[i].signature = 0L;		/* 0x0605b50 */
	    zio_b[i].disk_num = 0;		/* number of this disk */
	    zio_b[i].cdir_begins_on = 0;	/* number of the disk with the start of the central directory */
	    zio_b[i].on_this_disk = 0;		/* total number of entries in the central directory on this disk */
	    zio_b[i].total_in_cdir = 0;		/* total number of entries in the central directory */
	    zio_b[i].cdir_size = 0l;		/* size of the central directory */
	    zio_b[i].start_cdir = 0l;		/* offset to beginning of central director with respect to starting disk number */
	    zio_b[i].comment_len = 0;		/* size of comment for zip file */
	    zio_b[i].comment[0] = EOS;		/* comment string */
	}					/* end of for */
    }						/* end of if first time */

 /*
  * look for unused zip file handle 
  */
    for (i = 0; i < MAX_ZIOB; i++) {
	if (zio_b[i].fh == -1)			/* if this entry is not used */
	    break;				/* break out of for loop */
    }

    if (i >= MAX_ZIOB) {			/* if out of file handles */
	_ZERR = 4;				/* store error code */
	ret = -1;				/* set error flag */
	cprintf("\nno free handles.\n");
	goto done;				/* indicate error condition */
    }						/* end of if out of file handles */

    zh = i;					/* save zip handle  i is volatile */

    fh = open(name, O_RDONLY | O_RAW, 0);	/* try to open file */
    if (fh == -1) {				/* if file not found */
	_ZERR = _OSERR;				/* save error code */
	ret = -1;				/* flag error */
	poserr(name);
	goto done;				/* jump to exit */
    }						/* end of if file not opened */

 /*
  * look for end of central dir marker 
  */
    for (pos = lseek(fh, -(long) sizeof(end_central_zip), 2); pos != -1; pos = lseek(fh, pos - 1L, 0)) {
	i = read(fh, (char *) &sig, sizeof(unsigned long));	/* read an MsDos unsigned long, 4 bytes */
	if (i != sizeof(unsigned long)) {	/* bad read */
	    _ZERR = _OSERR;			/* save MsDos error code */
	    ret = -1;				/* flag an error */
	    poserr("read");
	    goto done;				/* jump to exit */
	}					/* end of read error */

	if (sig == END_CEN_SIG) {		/* hey we found the beginning of an end of central dir record */
	    lseek(fh, pos, 0);			/* backup */
	    break;				/* leave for loop */
	}					/* end of if found */
    }						/* end of for not at beginning of file */

    if (pos == -1L) {				/* assume illegal seek means we did not find the end of cen dir signature */
	_ZERR = _OSERR;				/* save the MsDos error code */
	ret = -1;				/* flag error */
	poserr("before beginning");
	goto done;				/* jump to exit */
    }						/* end of if seek error */

    i = read(fh, (char *) &zio_b[zh].signature, sizeof(end_central_zip));	/* read end of central dir record  */
    if (i != sizeof(end_central_zip)) {		/* if bad read */
	_ZERR = _OSERR;				/* save MsDos error code */
	ret = -1;				/* flag error */
	poserr("end of cdir read");
	goto done;				/* jump to exit */
    }						/* end of if bad read */

    if (zio_b[zh].comment_len > 0) {		/* if comment, read comment */
	size = (zio_b[zh].comment_len < (COMMENT_SIZE - 2)) ? zio_b[zh].comment_len : COMMENT_SIZE - 2;
	i = read(fh, (char *) zio_b[zh].comment, size);	/* read end of central dir record  */
	zio_b[zh].comment[size] = EOS;		/* terminate with EOS */
	if (i != size) {			/* if bad read */
	    _ZERR = _OSERR;			/* save MsDos error code */
	    ret = -1;				/* flag error */
	    cprintf("\nread zio_b[%d].comment\n", zh);
	    poserr("read comment");
	    goto done;
	}					/* end of if error reading comment */
    }						/* end of if comment */

    stccpy(zio_b[zh].name, name, NAME_SIZE);	/* save full MsDos path of zip file */
    zio_b[zh].fh = fh;				/* save file handle */
    ret = zh;					/* all is ok */

done:
    if (ret < 0) {				/* if error */
	if (fh != -1)				/* if file is open */
	    close(fh);				/* close it */
    }
    return (ret);
}						/* end of zopen */


int
zclose(zh)
int             zh;
{
int             ret;

    _ZERR = 0;					/* clear error flag */

    if ((zh < -1) || (zh > MAX_ZIOB - 1)) {	/* if invalid handle */
	_ZERR = 6;
	ret = -1;
	goto done;
    }

    if ((zh == -1) || (zio_b[zh].fh == -1)) {	/* if file is closed */
	_ZERR = 31;
	ret = -1;
	goto done;
    }

 /* most information is left intact, file handle is set to (-1) to indicate entry is free, this is the only necessity. */
 /* name and comment strings are made zero length, only first byte is distroyed */

    ret = close(zio_b[zh].fh);			/* close the file, save result */
    zio_b[zh].fh = -1;				/* return this entry to freelist */
    zio_b[zh].name[0] = EOS;			/* trunk name */
    zio_b[zh].comment[0] = EOS;			/* ""  comment */
    _ZERR = _OSERR;				/* save oserr */

done:
    return (ret);
}						/* end of zclose */

#ifdef TEST

 /*
  * print central_zip struct pointed to by cz 
  */

void
pcz(cz)
central_zip    *cz;
{
    cprintf("\nDump of central_zip record at [%p]\n", cz);
    cprintf("    signature           -:%lx\n", cz->signature);	/* 0x02014b50 */
    cprintf("    ver_made_by         -:%x\n", cz->ver_made_by);
    cprintf("    ver_to_extract      -:%x\n", cz->ver_to_extract);
    cprintf("    bit_flag.encrypted   -:%u\n", (unsigned int) cz->bit_flag.encrypted);
    cprintf("    bit_flag.fill        -:%u\n", (unsigned int) cz->bit_flag.fill);
    cprintf("    bit_flag.reserved    -:%u\n", (unsigned int) cz->bit_flag.reserved);
    cprintf("    comp_method         -:%u\n", cz->comp_method);
    cprintf("    time                -:%u\n", cz->time);
    cprintf("    date                -:%u\n", cz->date);
    cprintf("    crc                 -:%lu\n", cz->crc);
    cprintf("    compressed_size     -:%lu\n", cz->compressed_size);
    cprintf("    uncompressed_size   -:%lu\n", cz->uncompressed_size);
    cprintf("    name_len            -:%u\n", cz->name_len);
    cprintf("    caca_len            -:%u\n", cz->caca_len);
    cprintf("    comment_len         -:%u\n", cz->comment_len);	/* file comment length */
    cprintf("    disk_num_start      -:%u\n", cz->disk_num_start);	/* number of disk holding this files beginning */
    cprintf("    in_attrib.ascii     -:%u\n", (unsigned int) cz->in_attrib.ascii);	/* zip internal file attributes */
    cprintf("    in_attrib.caca      -:%u\n", (unsigned int) cz->in_attrib.caca);	/* zip internal file attributes */
    cprintf("    extern_attrib       -:%lu\n", cz->extern_attrib);	/* system dependent, low order byte is MsDos */
    cprintf("    local_header_offset -:%lu\n", cz->local_header_offset);	/* offset on first disk to local header */
    cprintf("end.\n");
    return;
}						/* end of pcz */




 /*
  * print local_zip struct pointed to by lz 
  */

void
plz(lz)
local_zip      *lz;
{
    cprintf("\nDump of local_zip record at [%p]\n", lz);
    cprintf("    signature           -:%lx\n", lz->signature);	/* 0x04034b50 */
    cprintf("    ver_to_extract      -:%x\n", lz->ver_to_extract);
    cprintf("    bit_flag.encrypted   -:%u\n", (unsigned int) lz->bit_flag.encrypted);
    cprintf("    bit_flag.fill        -:%u\n", (unsigned int) lz->bit_flag.fill);
    cprintf("    bit_flag.reserved    -:%u\n", (unsigned int) lz->bit_flag.reserved);
    cprintf("    comp_method         -:%u\n", lz->comp_method);
    cprintf("    time                -:%u\n", lz->time);
    cprintf("    date                -:%u\n", lz->date);
    cprintf("    crc                 -:%lu\n", lz->crc);
    cprintf("    compressed_size     -:%lu\n", lz->compressed_size);
    cprintf("    uncompressed_size   -:%lu\n", lz->uncompressed_size);
    cprintf("    name_len            -:%u\n", lz->name_len);
    cprintf("    caca_len            -:%u\n", lz->caca_len);
    cprintf("end.\n");
    return;
}						/* end of plz */


void
pecz(zh)
int             zh;
{
    cprintf("\nDump of zio_b[%d]\n", zh);
    cprintf("    name           -:%s\n", zio_b[zh].name);
    cprintf("    fh             -:%d\n", zio_b[zh].fh);
    cprintf("    signature      -:%lx\n", zio_b[zh].signature);
    cprintf("    disk_num       -:%u\n", zio_b[zh].disk_num);
    cprintf("    cdir_begins_on -:%u\n", zio_b[zh].cdir_begins_on);
    cprintf("    on_this_disk   -:%u\n", zio_b[zh].on_this_disk);
    cprintf("    total_in_cdir  -:%u\n", zio_b[zh].total_in_cdir);
    cprintf("    cdir_size      -:%lu\n", zio_b[zh].cdir_size);
    cprintf("    start_dir      -:%lu\n", zio_b[zh].start_cdir);
    cprintf("    comment_len    -:%d\n", zio_b[zh].comment_len);
    cprintf("    comment        -:%s\n", zio_b[zh].comment);
    cprintf("end.\n");
    return;
}						/* end of print end central zip */



void
pzi(zi)
ZIPINFO        *zi;
{
    cprintf("\nDump of ZIPINFO record at [%p]\n", zi);
    cprintf("    ZIO                 -:%p\n", zi->zptr);	/* */
    cprintf("    pattern             -:%s\n", zi->pattern);
    cprintf("    next_offset         -:%lu\n", zi->next_offset);	/* offset of THIS central dir record */
    cprintf("    ver_made_by         -:%x\n", zi->ver_made_by);
    cprintf("    ver_to_extract      -:%x\n", zi->ver_to_extract);
    cprintf("    bit_flag.encrypted   -:%u\n", (unsigned int) zi->bit_flag.encrypted);
    cprintf("    bit_flag.fill        -:%u\n", (unsigned int) zi->bit_flag.fill);
    cprintf("    bit_flag.reserved    -:%u\n", (unsigned int) zi->bit_flag.reserved);
    cprintf("    comp_method         -:%u\n", zi->comp_method);
    cprintf("    time                -:%u\n", zi->time);
    cprintf("    date                -:%u\n", zi->date);
    cprintf("    crc                 -:%lu\n", zi->crc);
    cprintf("    compressed_size     -:%lu\n", zi->compressed_size);
    cprintf("    uncompressed_size   -:%lu\n", zi->uncompressed_size);
    cprintf("    disk_num_start      -:%u\n", zi->disk_num_start);	/* number of disk holding this files beginning */
    cprintf("    in_attrib.ascii     -:%u\n", (unsigned int) zi->in_attrib.ascii);	/* zip internal file attributes */
    cprintf("    in_attrib.caca      -:%u\n", (unsigned int) zi->in_attrib.caca);	/* zip internal file attributes */
    cprintf("    extern_attrib       -:%lu\n", zi->extern_attrib);	/* system dependent, low order byte is MsDos */
    cprintf("    local_header_offset -:%lu\n", zi->local_header_offset);	/* offset on first disk to local header */
    cprintf("    name                -:%s\n", zi->name);
    cprintf("    comment             -:%s\n", zi->comment);
    cprintf("end.\n");
    return;
}						/* end of pzi */

#endif


 /*
  * End of file ZFIND.C 
  */
